@@ -27,7 +27,7 @@ impl<'de> Deserializer<'de> {
     where F: FnOnce(&'de UnwrappedIOValue) -> Option<T>
     {
         f(self.input.value()).ok_or_else(
-            || Error::Expected(k, Received::ReceivedOtherValue(self.input.clone())))
+            || Error::Expected(k, Received::ReceivedOtherValue(format!("{:?}", self.input))))
     }
 }
 
