@@ -5,7 +5,6 @@ use std::io;
 use std::ops::DerefMut;
 use super::constants::Tag;
 use super::super::DomainEncode;
-use super::super::Embeddable;
 use super::super::IOValue;
 use super::super::IOValueDomainCodec;
 use super::super::NestedValue;
@@ -17,7 +16,8 @@ use super::super::writer::{Writer, CompoundWriter, varint};
 pub struct PackedWriter<W: io::Write>(Suspendable<W>);
 
 impl PackedWriter<&mut Vec<u8>> {
-    pub fn encode<D: Embeddable, N: NestedValue<D>, Enc: DomainEncode<D>>(
+    #[inline(always)]
+    pub fn encode<N: NestedValue, Enc: DomainEncode<N::Embedded>>(
         enc: &mut Enc,
         v: &N,
     ) -> io::Result<Vec<u8>> {
@@ -26,24 +26,29 @@ impl PackedWriter<&mut Vec<u8>> {
         Ok(buf)
     }
 
+    #[inline(always)]
     pub fn encode_iovalue(v: &IOValue) -> io::Result<Vec<u8>> {
         Self::encode(&mut IOValueDomainCodec, v)
     }
 }
 
 impl<W: io::Write> PackedWriter<W> {
+    #[inline(always)]
     pub fn new(write: W) -> Self {
         PackedWriter(Suspendable::new(write))
     }
 
+    #[inline(always)]
     pub fn w(&mut self) -> &mut W {
         self.0.deref_mut()
     }
 
+    #[inline(always)]
     pub fn write_byte(&mut self, b: u8) -> io::Result<()> {
         self.w().write_all(&[b])
     }
 
+    #[inline(always)]
     pub fn write_medium_integer(&mut self, bs: &[u8]) -> io::Result<()> {
         let count: u8 = bs.len().try_into().unwrap();
         if !(1..=16).contains(&count) { panic!("Invalid medium_integer count: {}", count) }
@@ -51,16 +56,19 @@ impl<W: io::Write> PackedWriter<W> {
         self.w().write_all(bs)
     }
 
+    #[inline(always)]
     pub fn write_atom(&mut self, tag: Tag, bs: &[u8]) -> io::Result<()> {
         self.write_byte(tag.into())?;
         varint(&mut self.w(), bs.len().try_into().unwrap())?;
         self.w().write_all(bs)
     }
 
+    #[inline(always)]
     pub fn suspend(&mut self) -> Self {
         PackedWriter(self.0.suspend())
     }
 
+    #[inline(always)]
     pub fn resume(&mut self, other: Self) {
         self.0.resume(other.0)
     }
@@ -69,30 +77,37 @@ impl<W: io::Write> PackedWriter<W> {
 pub struct BinaryOrderWriter(Vec<Vec<u8>>);
 
 impl BinaryOrderWriter {
+    #[inline(always)]
     fn new() -> Self {
         BinaryOrderWriter(vec![vec![]])
     }
 
+    #[inline(always)]
     fn pop(&mut self) -> PackedWriter<Vec<u8>> {
         PackedWriter::new(self.0.pop().unwrap())
     }
 
+    #[inline(always)]
     fn push(&mut self, w: PackedWriter<Vec<u8>>) {
         self.0.push(w.0.take())
     }
 
+    #[inline(always)]
     fn items(&self) -> &Vec<Vec<u8>> {
         &self.0
     }
 
+    #[inline(always)]
     fn items_mut(&mut self) -> &mut Vec<Vec<u8>> {
         &mut self.0
     }
 
+    #[inline(always)]
     fn buffer(&mut self) -> &mut Vec<u8> {
         self.0.last_mut().unwrap()
     }
 
+    #[inline(always)]
     fn finish<W: WriteWriter>(mut self, w: &mut W) -> io::Result<()> {
         if !self.buffer().is_empty() { panic!("Missing final boundary()"); }
         self.items_mut().pop();
@@ -107,18 +122,22 @@ impl BinaryOrderWriter {
 
 pub trait WriteWriter: Writer {
     fn write_raw_bytes(&mut self, v: &[u8]) -> io::Result<()>;
+
+    #[inline(always)]
     fn write_tag(&mut self, tag: Tag) -> io::Result<()> {
         self.write_raw_bytes(&[tag.into()])
     }
 }
 
 impl<W: io::Write> WriteWriter for PackedWriter<W> {
+    #[inline(always)]
     fn write_raw_bytes(&mut self, v: &[u8]) -> io::Result<()> {
         self.w().write_all(v)
     }
 }
 
 impl WriteWriter for BinaryOrderWriter {
+    #[inline(always)]
     fn write_raw_bytes(&mut self, v: &[u8]) -> io::Result<()> {
         use io::Write;
         self.buffer().write_all(v)
@@ -126,7 +145,7 @@ impl WriteWriter for BinaryOrderWriter {
 }
 
 impl<W: io::Write> CompoundWriter for PackedWriter<W> {
-    #[inline]
+    #[inline(always)]
     fn boundary(&mut self, b: &B::Type) -> io::Result<()> {
         if let Some(B::Item::Annotation) = b.opening {
             self.write_tag(Tag::Annotation)?;
@@ -136,7 +155,7 @@ impl<W: io::Write> CompoundWriter for PackedWriter<W> {
 }
 
 impl CompoundWriter for BinaryOrderWriter {
-    #[inline]
+    #[inline(always)]
     fn boundary(&mut self, b: &B::Type) -> io::Result<()> {
         match b.closing {
             Some(B::Item::DictionaryValue) |
@@ -153,7 +172,7 @@ impl CompoundWriter for BinaryOrderWriter {
 
 macro_rules! binary_order_writer_method {
     (mut $n:ident ($($argname:ident : $argty:ty),*) -> $retty:ty) =>
-        (fn $n (&mut self, $($argname : $argty),*) -> $retty {
+        (#[inline(always)] fn $n (&mut self, $($argname : $argty),*) -> $retty {
             (&mut PackedWriter::new(self.buffer())).$n($($argname),*)
         });
 }
@@ -166,9 +185,11 @@ impl Writer for BinaryOrderWriter {
     type DictWriter = BinaryOrderWriter;
     type EmbeddedWriter = PackedWriter<Vec<u8>>;
 
+    #[inline(always)]
     fn start_annotations(&mut self) -> io::Result<Self::AnnWriter> {
         Ok(self.pop())
     }
+    #[inline(always)]
     fn end_annotations(&mut self, ann: Self::AnnWriter) -> io::Result<()> {
         self.push(ann);
         Ok(())
@@ -195,44 +216,54 @@ impl Writer for BinaryOrderWriter {
     binary_order_writer_method!(mut write_bytes(v: &[u8]) -> io::Result<()>);
     binary_order_writer_method!(mut write_symbol(v: &str) -> io::Result<()>);
 
+    #[inline(always)]
     fn start_record(&mut self, _field_count: Option<usize>) -> io::Result<Self::RecWriter> {
         self.write_tag(Tag::Record)?;
         Ok(self.pop())
     }
+    #[inline(always)]
     fn end_record(&mut self, rec: Self::RecWriter) -> io::Result<()> {
         self.push(rec);
         self.write_tag(Tag::End)
     }
 
+    #[inline(always)]
     fn start_sequence(&mut self, _item_count: Option<usize>) -> io::Result<Self::SeqWriter> {
         self.write_tag(Tag::Sequence)?;
         Ok(self.pop())
     }
+    #[inline(always)]
     fn end_sequence(&mut self, seq: Self::SeqWriter) -> io::Result<()> {
         self.push(seq);
         self.write_tag(Tag::End)
     }
 
+    #[inline(always)]
     fn start_set(&mut self, _item_count: Option<usize>) -> io::Result<Self::SetWriter> {
         self.write_tag(Tag::Set)?;
         Ok(BinaryOrderWriter::new())
     }
+    #[inline(always)]
     fn end_set(&mut self, set: Self::SetWriter) -> io::Result<()> {
         set.finish(self)
     }
 
+    #[inline(always)]
     fn start_dictionary(&mut self, _entry_count: Option<usize>) -> io::Result<Self::DictWriter> {
         self.write_tag(Tag::Dictionary)?;
         Ok(BinaryOrderWriter::new())
     }
+    #[inline(always)]
     fn end_dictionary(&mut self, dict: Self::DictWriter) -> io::Result<()> {
         dict.finish(self)
     }
 
+    #[inline(always)]
     fn start_embedded(&mut self) -> io::Result<Self::EmbeddedWriter> {
         self.write_tag(Tag::Embedded)?;
         Ok(self.pop())
     }
+    #[inline(always)]
     fn end_embedded(&mut self, ptr: Self::EmbeddedWriter) -> io::Result<()> {
         self.push(ptr);
         Ok(())
@@ -255,49 +286,59 @@ impl<W: io::Write> Writer for PackedWriter<W>
     type DictWriter = BinaryOrderWriter;
     type EmbeddedWriter = Self;
 
+    #[inline(always)]
     fn start_annotations(&mut self) -> io::Result<Self::AnnWriter> {
         Ok(self.suspend())
     }
 
+    #[inline(always)]
     fn end_annotations(&mut self, ann: Self::AnnWriter) -> io::Result<()> {
         self.resume(ann);
         Ok(())
     }
 
+    #[inline(always)]
     fn write_bool(&mut self, v: bool) -> io::Result<()> {
         self.write_tag(if v { Tag::True } else { Tag::False })
     }
 
+    #[inline(always)]
     fn write_f32(&mut self, v: f32) -> io::Result<()> {
         self.write_tag(Tag::Float)?;
         self.write_raw_bytes(&u32::to_be_bytes(f32::to_bits(v)))
     }
 
+    #[inline(always)]
     fn write_f64(&mut self, v: f64) -> io::Result<()> {
         self.write_tag(Tag::Double)?;
         self.write_raw_bytes(&u64::to_be_bytes(f64::to_bits(v)))
     }
 
+    #[inline(always)]
     fn write_i8(&mut self, v: i8) -> io::Result<()> {
         if v >= -3 && v <= 12 { return self.write_tag(Tag::SmallInteger(v)) }
         self.write_medium_integer(&[v as u8])
     }
 
+    #[inline(always)]
     fn write_u8(&mut self, v: u8) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i8(w) }
         self.write_medium_integer(&[0, v])
     }
 
+    #[inline(always)]
     fn write_i16(&mut self, v: i16) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i8(w) }
         self.write_medium_integer(&[(v >> 8) as u8, (v & 255) as u8])
     }
 
+    #[inline(always)]
     fn write_u16(&mut self, v: u16) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i16(w) }
         self.write_medium_integer(&[0, (v >> 8) as u8, (v & 255) as u8])
     }
 
+    #[inline(always)]
     fn write_i32(&mut self, v: i32) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i16(w) }
         if fits_in_bytes!(v, 3) {
@@ -311,6 +352,7 @@ impl<W: io::Write> Writer for PackedWriter<W>
                                     (v & 255) as u8])
     }
 
+    #[inline(always)]
     fn write_u32(&mut self, v: u32) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i32(w) }
         self.write_medium_integer(&[0,
@@ -320,6 +362,7 @@ impl<W: io::Write> Writer for PackedWriter<W>
                                     (v & 255) as u8])
     }
 
+    #[inline(always)]
     fn write_i64(&mut self, v: i64) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i32(w) }
         if fits_in_bytes!(v, 5) {
@@ -356,6 +399,7 @@ impl<W: io::Write> Writer for PackedWriter<W>
                                     (v & 255) as u8])
     }
 
+    #[inline(always)]
     fn write_u64(&mut self, v: u64) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i64(w) }
         self.write_medium_integer(&[0,
@@ -369,6 +413,7 @@ impl<W: io::Write> Writer for PackedWriter<W>
                                     (v & 255) as u8])
     }
 
+    #[inline(always)]
     fn write_i128(&mut self, v: i128) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i64(w) }
         let bs: [u8; 16] = v.to_be_bytes();
@@ -382,6 +427,7 @@ impl<W: io::Write> Writer for PackedWriter<W>
         self.write_medium_integer(&bs)
     }
 
+    #[inline(always)]
     fn write_u128(&mut self, v: u128) -> io::Result<()> {
         if let Ok(w) = v.try_into() { return self.write_i128(w) }
         let bs: [u8; 16] = v.to_be_bytes();
@@ -391,6 +437,7 @@ impl<W: io::Write> Writer for PackedWriter<W>
         self.write_raw_bytes(&bs)
     }
 
+    #[inline(always)]
     fn write_int(&mut self, v: &BigInt) -> io::Result<()> {
         match v.to_i8() {
             Some(n) => self.write_i8(n),
@@ -403,61 +450,74 @@ impl<W: io::Write> Writer for PackedWriter<W>
         }
     }
 
+    #[inline(always)]
     fn write_string(&mut self, v: &str) -> io::Result<()> {
         self.write_atom(Tag::String, v.as_bytes())
     }
 
+    #[inline(always)]
     fn write_bytes(&mut self, v: &[u8]) -> io::Result<()> {
         self.write_atom(Tag::ByteString, v)
     }
 
+    #[inline(always)]
     fn write_symbol(&mut self, v: &str) -> io::Result<()> {
         self.write_atom(Tag::Symbol, v.as_bytes())
     }
 
+    #[inline(always)]
     fn start_record(&mut self, _field_count: Option<usize>) -> io::Result<Self::RecWriter> {
         self.write_tag(Tag::Record)?;
         Ok(self.suspend())
     }
 
+    #[inline(always)]
     fn end_record(&mut self, rec: Self::RecWriter) -> io::Result<()> {
         self.resume(rec);
         self.write_tag(Tag::End)
     }
 
+    #[inline(always)]
     fn start_sequence(&mut self, _item_count: Option<usize>) -> io::Result<Self::SeqWriter> {
         self.write_tag(Tag::Sequence)?;
         Ok(self.suspend())
     }
 
+    #[inline(always)]
     fn end_sequence(&mut self, seq: Self::SeqWriter) -> io::Result<()> {
         self.resume(seq);
         self.write_tag(Tag::End)
     }
 
+    #[inline(always)]
     fn start_set(&mut self, _item_count: Option<usize>) -> io::Result<Self::SetWriter> {
         self.write_tag(Tag::Set)?;
         Ok(BinaryOrderWriter::new())
     }
 
+    #[inline(always)]
     fn end_set(&mut self, set: Self::SetWriter) -> io::Result<()> {
         set.finish(self)
     }
 
+    #[inline(always)]
     fn start_dictionary(&mut self, _entry_count: Option<usize>) -> io::Result<Self::DictWriter> {
         self.write_tag(Tag::Dictionary)?;
         Ok(BinaryOrderWriter::new())
     }
 
+    #[inline(always)]
     fn end_dictionary(&mut self, dict: Self::DictWriter) -> io::Result<()> {
         dict.finish(self)
     }
 
+    #[inline(always)]
     fn start_embedded(&mut self) -> io::Result<Self::EmbeddedWriter> {
         self.write_tag(Tag::Embedded)?;
         Ok(self.suspend())
     }
 
+    #[inline(always)]
     fn end_embedded(&mut self, ann: Self::EmbeddedWriter) -> io::Result<()> {
         self.resume(ann);
         Ok(())
