@@ -8,7 +8,6 @@ use super::CompoundClass;
 use super::DomainDecode;
 use super::DomainParse;
 use super::Double;
-use super::Embeddable;
 use super::Float;
 use super::IOValue;
 use super::IOValueDomainCodec;
@@ -19,14 +18,14 @@ use super::signed_integer::SignedInteger;
 
 pub type ReaderResult<T> = std::result::Result<T, error::Error>;
 
-pub enum Token<D: Embeddable, N: NestedValue<D>> {
-    Embedded(D),
+pub enum Token<N: NestedValue> {
+    Embedded(N::Embedded),
     Atom(N),
     Compound(CompoundClass),
     End,
 }
 
-pub trait Reader<'de, D: Embeddable, N: NestedValue<D>> {
+pub trait Reader<'de, N: NestedValue> {
     fn next(&mut self, read_annotations: bool) -> io::Result<Option<N>>;
     fn open_record(&mut self, arity: Option<usize>) -> ReaderResult<B::Type>;
     fn open_sequence_or_set(&mut self) -> ReaderResult<B::Item>;
@@ -45,8 +44,8 @@ pub trait Reader<'de, D: Embeddable, N: NestedValue<D>> {
     fn mark(&mut self) -> io::Result<Self::Mark>;
     fn restore(&mut self, mark: &Self::Mark) -> io::Result<()>;
 
-    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<D, N>>;
-    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<D, N>)>;
+    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<N>>;
+    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<N>)>;
 
     //---------------------------------------------------------------------------
 
@@ -131,7 +130,7 @@ pub trait Reader<'de, D: Embeddable, N: NestedValue<D>> {
         }
     }
 
-    fn configured(self, read_annotations: bool) -> ConfiguredReader<'de, D, N, Self>
+    fn configured(self, read_annotations: bool) -> ConfiguredReader<'de, N, Self>
     where
         Self: std::marker::Sized
     {
@@ -159,10 +158,7 @@ pub trait Reader<'de, D: Embeddable, N: NestedValue<D>> {
     }
 }
 
-impl<'r, 'de, D: Embeddable, N: NestedValue<D>, R: Reader<'de, D, N>>
-    Reader<'de, D, N>
-    for &'r mut R
-{
+impl<'r, 'de, N: NestedValue, R: Reader<'de, N>> Reader<'de, N> for &'r mut R {
     fn next(&mut self, read_annotations: bool) -> io::Result<Option<N>> {
         (*self).next(read_annotations)
     }
@@ -213,11 +209,11 @@ impl<'r, 'de, D: Embeddable, N: NestedValue<D>, R: Reader<'de, D, N>>
         (*self).restore(mark)
     }
 
-    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<D, N>> {
+    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<N>> {
         (*self).next_token(read_embedded_annotations)
     }
 
-    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<D, N>)> {
+    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<N>)> {
         (*self).next_annotations_and_token()
     }
 }
@@ -233,30 +229,30 @@ pub trait BinarySource<'de>: Sized {
     fn readbytes(&mut self, count: usize) -> io::Result<Cow<'de, [u8]>>;
     fn readbytes_into(&mut self, bs: &mut [u8]) -> io::Result<()>;
 
-    fn packed<D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>>(
+    fn packed<N: NestedValue, Dec: DomainDecode<N::Embedded>>(
         &mut self,
         decode_embedded: Dec,
-    ) -> super::PackedReader<'de, '_, D, N, Dec, Self> {
+    ) -> super::PackedReader<'de, '_, N, Dec, Self> {
         super::PackedReader::new(self, decode_embedded)
     }
 
     fn packed_iovalues(&mut self) ->
-        super::PackedReader<'de, '_, IOValue, IOValue, IOValueDomainCodec, Self>
+        super::PackedReader<'de, '_, IOValue, IOValueDomainCodec, Self>
     {
         self.packed(IOValueDomainCodec)
     }
 
-    fn text<D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>>(
+    fn text<N: NestedValue, Dec: DomainParse<N::Embedded>>(
         &mut self,
         decode_embedded: Dec,
-    ) -> super::TextReader<'de, '_, D, Dec, Self> {
+    ) -> super::TextReader<'de, '_, N::Embedded, Dec, Self> {
         super::TextReader::new(self, decode_embedded)
     }
 
     fn text_iovalues(&mut self) ->
         super::TextReader<'de, '_, IOValue, ViaCodec<IOValueDomainCodec>, Self>
     {
-        self.text::<_, IOValue, _>(ViaCodec::new(IOValueDomainCodec))
+        self.text::<IOValue, _>(ViaCodec::new(IOValueDomainCodec))
     }
 }
 
@@ -266,6 +262,7 @@ pub struct IOBinarySource<R: io::Read + io::Seek> {
 }
 
 impl<R: io::Read + io::Seek> IOBinarySource<R> {
+    #[inline(always)]
     pub fn new(read: R) -> Self {
         IOBinarySource { read, buf: None }
     }
@@ -274,22 +271,26 @@ impl<R: io::Read + io::Seek> IOBinarySource<R> {
 impl<'de, R: io::Read + io::Seek> BinarySource<'de> for IOBinarySource<R> {
     type Mark = u64;
 
+    #[inline(always)]
     fn mark(&mut self) -> io::Result<Self::Mark> {
         Ok(self.read.stream_position()? - (if self.buf.is_some() { 1 } else { 0 }))
     }
 
+    #[inline(always)]
     fn restore(&mut self, mark: &Self::Mark) -> io::Result<()> {
         self.read.seek(io::SeekFrom::Start(*mark))?;
         self.buf = None;
         Ok(())
     }
 
+    #[inline(always)]
     fn skip(&mut self) -> io::Result<()> {
         if self.buf.is_none() { unreachable!(); }
         self.buf = None;
         Ok(())
     }
 
+    #[inline(always)]
     fn peek(&mut self) -> io::Result<u8> {
         match self.buf {
             Some(b) => Ok(b),
@@ -307,6 +308,7 @@ impl<'de, R: io::Read + io::Seek> BinarySource<'de> for IOBinarySource<R> {
         }
     }
 
+    #[inline(always)]
     fn readbytes(&mut self, count: usize) -> io::Result<Cow<'de, [u8]>> {
         if self.buf.is_some() { unreachable!(); }
         let mut bs = vec![0; count];
@@ -314,6 +316,7 @@ impl<'de, R: io::Read + io::Seek> BinarySource<'de> for IOBinarySource<R> {
         Ok(Cow::Owned(bs))
     }
 
+    #[inline(always)]
     fn readbytes_into(&mut self, bs: &mut [u8]) -> io::Result<()> {
         if self.buf.is_some() { unreachable!(); }
         self.read.read_exact(bs)
@@ -326,6 +329,7 @@ pub struct BytesBinarySource<'de> {
 }
 
 impl<'de> BytesBinarySource<'de> {
+    #[inline(always)]
     pub fn new(bytes: &'de [u8]) -> Self {
         BytesBinarySource { bytes, index: 0 }
     }
@@ -334,21 +338,25 @@ impl<'de> BytesBinarySource<'de> {
 impl<'de> BinarySource<'de> for BytesBinarySource<'de> {
     type Mark = usize;
 
+    #[inline(always)]
     fn mark(&mut self) -> io::Result<Self::Mark> {
         Ok(self.index)
     }
 
+    #[inline(always)]
     fn restore(&mut self, mark: &Self::Mark) -> io::Result<()> {
         self.index = *mark;
         Ok(())
     }
 
+    #[inline(always)]
     fn skip(&mut self) -> io::Result<()> {
         if self.index >= self.bytes.len() { unreachable!(); }
         self.index += 1;
         Ok(())
     }
 
+    #[inline(always)]
     fn peek(&mut self) -> io::Result<u8> {
         if self.index >= self.bytes.len() {
             Err(io_eof())
@@ -357,6 +365,7 @@ impl<'de> BinarySource<'de> for BytesBinarySource<'de> {
         }
     }
 
+    #[inline(always)]
     fn readbytes(&mut self, count: usize) -> io::Result<Cow<'de, [u8]>> {
         if self.index + count > self.bytes.len() {
             Err(io_eof())
@@ -367,6 +376,7 @@ impl<'de> BinarySource<'de> for BytesBinarySource<'de> {
         }
     }
 
+    #[inline(always)]
     fn readbytes_into(&mut self, bs: &mut [u8]) -> io::Result<()> {
         let count = bs.len();
         if self.index + count > self.bytes.len() {
@@ -379,13 +389,13 @@ impl<'de> BinarySource<'de> for BytesBinarySource<'de> {
     }
 }
 
-pub struct ConfiguredReader<'de, D: Embeddable, N: NestedValue<D>, R: Reader<'de, D, N>> {
+pub struct ConfiguredReader<'de, N: NestedValue, R: Reader<'de, N>> {
     pub reader: R,
     pub read_annotations: bool,
-    phantom: PhantomData<&'de (D, N)>,
+    phantom: PhantomData<&'de N>,
 }
 
-impl<'de, D: Embeddable, N: NestedValue<D>, R: Reader<'de, D, N>> ConfiguredReader<'de, D, N, R> {
+impl<'de, N: NestedValue, R: Reader<'de, N>> ConfiguredReader<'de, N, R> {
     pub fn new(reader: R) -> Self {
         reader.configured(true)
     }
@@ -399,10 +409,7 @@ impl<'de, D: Embeddable, N: NestedValue<D>, R: Reader<'de, D, N>> ConfiguredRead
     }
 }
 
-impl<'de, D: Embeddable, N: NestedValue<D>, R: Reader<'de, D, N>>
-    std::iter::Iterator
-    for ConfiguredReader<'de, D, N, R>
-{
+impl<'de, N: NestedValue, R: Reader<'de, N>> std::iter::Iterator for ConfiguredReader<'de, N, R> {
     type Item = io::Result<N>;
     fn next(&mut self) -> Option<Self::Item> {
         match self.reader.next(self.read_annotations) {
