@@ -734,6 +734,14 @@ impl<N: NestedValue> Value<N> {
         }
     }
 
+    #[inline(always)]
+    pub fn into_string(self) -> Option<String> {
+        match self {
+            Value::String(s) => Some(s),
+            _ => None,
+        }
+    }
+
     #[inline(always)]
     pub fn to_string(&self) -> Result<&String, Error> {
         self.as_string().ok_or_else(|| self.expected(ExpectedKind::String))
@@ -767,6 +775,14 @@ impl<N: NestedValue> Value<N> {
         }
     }
 
+    #[inline(always)]
+    pub fn into_bytestring(self) -> Option<Vec<u8>> {
+        match self {
+            Value::ByteString(bs) => Some(bs),
+            _ => None,
+        }
+    }
+
     #[inline(always)]
     pub fn to_bytestring(&self) -> Result<&Vec<u8>, Error> {
         self.as_bytestring().ok_or_else(|| self.expected(ExpectedKind::ByteString))
@@ -800,6 +816,14 @@ impl<N: NestedValue> Value<N> {
         }
     }
 
+    #[inline(always)]
+    pub fn into_symbol(self) -> Option<String> {
+        match self {
+            Value::Symbol(s) => Some(s),
+            _ => None,
+        }
+    }
+
     #[inline(always)]
     pub fn to_symbol(&self) -> Result<&String, Error> {
         self.as_symbol().ok_or_else(|| self.expected(ExpectedKind::Symbol))
@@ -1389,7 +1413,10 @@ impl<D: Embeddable> NestedValue for ArcValue<D> {
 
     #[inline(always)]
     fn value_owned(self) -> Value<Self> {
-        Arc::try_unwrap(self.0).unwrap_or_else(|_| panic!("value_owned on ArcValue with refcount greater than one")).1
+        match Arc::try_unwrap(self.0) {
+            Ok(AnnotatedValue(_anns, v)) => v,
+            Err(r) => r.1.clone(),
+        }
     }
 }
 
